function ShamPoly3(fx,gx,xRange,Lb,Ub,runNum,sFxName,diaryFilename)
%SHAMPOLY3 Summary of this function goes here
%   Detailed explanation goes here
clc
global bDeleteIfExists
global bUseDiary
global xdata
global ydata
global order  
global ggx

warning('off','all')
if isempty(sFxName)
  sFxName = getFuncName(fx);
end
xdata = xRange';
ydata = xdata;
for i=1:length(xdata)
  ydata(i)=fx(xdata(i));
end
ggx = gx;

fprintf('Fitting %s in range (%f, %f)\n', sFxName, min(xdata),max(xdata));

options = optimoptions('particleswarm', 'Display', 'iter');
[x,psAICc] = particleswarm(@optimFunc,4,Lb,Ub,options);
A = x(1);
B = x(2);
C = x(3);
order = round(x(4));
if bUseDiary
  diaryFilename = strrep(diaryFilename, ".txt", strcat("_", num2str(order),"_run", num2str(runNum),".txt"));
  if exist(diaryFilename, 'file')==2
    if bDeleteIfExists
      delete(diaryFilename);
    else
      return;
    end
  end
end
X = [];
for i=1:order
  xs = xdata.^gx(i,A,B,C);
  X = [X;xs'];
end
X = X';
lm = fitlm(X,ydata);

if bUseDiary 
  diary(diaryFilename)
end
fprintf('Fitting %s in range (%f, %f)\n', sFxName, min(xdata),max(xdata));
sp = getFuncName(gx);
fprintf('Shammas polynomial power is %s\n', sp);
format long
disp(lm);
anva = anova(lm,'summary');
disp(anva);
format short
fprintf('A = %f, B = %f, C = %f\n', A, B, C);
lstPwr = [];
for i=1:order
  lstPwr = [lstPwr,gx(i,A,B,C)];
end
fprintf('List of powers: [');
for i=1:order-1
 fprintf('%f, ', lstPwr(i));
end
fprintf('%f]\n', lstPwr(order));
fprintf('Fitting %s in range (%f, %f)\n', sFxName, min(xdata),max(xdata));
n = length(xdata);
sumsqr = 0;
for i=1:n
  yc = lm.Coefficients{1,1};
  for j=2:length(lm.Coefficients{:,1})
    yc = yc + lm.Coefficients{j,1} * xdata(i)^lstPwr(j-1);
  end
  sumsqr = sumsqr + (ydata(i) - yc)^2;
end  
k = order + 1;
fprintf('MSS of errors squared = %e\n', sqrt(sumsqr)/n);
fprintf('Corrected MSS of errors squared = %e\n', sqrt(sumsqr)/n*sqrt(3));
AIC = lm.ModelCriterion.AIC;
AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
fprintf('Particle swarm AICc =  %e\n', psAICc);
fprintf('AIC =  %e\n', AIC);
fprintf('AICc =  %e\n', AICc);

if bUseDiary
  diary off
end
end

function AICc = optimFunc(x)
  global xdata
  global ydata
  global order
  global ggx
  
  A = x(1);
  B = x(2);
  C = x(3);
  order = round(x(4));
  X = [];
  for i=1:order
    xs = xdata.^ggx(i,A,B,C);
    X = [X;xs'];
  end
  X = X';
  lm = fitlm(X,ydata);

  n = length(xdata);
  k = order + 1;
  AIC = lm.ModelCriterion.AIC;
  AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
  if isinf(AICc), AICc = -1e+99; end
end

function sFx = getFuncName(fx)
  sFx = func2str(fx);
  if sFx(1:2)=="@("
    i = strfind(sFx,")");
    sFx = sFx(i(1)+1:end);
  elseif sFx(1)=="@"
    sFx = strcat(sFx(2:end),".m");
  else
    % return sFx as is
  end
end